# -*- coding: utf-8 -*-
import json
from pathlib import Path
from typing import Union

import cv2
import numpy as np

from line_image_extractor.image_utils import (
    DESKEW,
    SKEW,
    WHITE,
    BoundingBox,
    Extraction,
    deskew_image,
    extract_bbox_image,
    extract_min_area_rect_image,
    extract_polygon_image,
    logger,
    polygon_to_bbox,
    resize,
    rotate_and_trim,
)


def extract(
    img: np.ndarray,
    polygon: np.ndarray,
    bbox: BoundingBox,
    extraction_mode: Extraction,
    max_deskew_angle: int = None,
    skew_angle: int = None,
    grayscale: bool = False,
) -> np.ndarray:
    """
    This is the main function to be used for extracting line images in tools/workers.

    It extracts either a bounding box image, a polygon or a minimum area rectangle.

    Also polygon and minimum area rectangles can be deskewed or skewed.

    :param img: Input page image (read with cv2(
    :param polygon: the polygon of the line to be extracted
    :param bbox: the bounding box of the line to be extracted
    :param extraction_mode: which mode of extraction to be used
    :param max_deskew_angle: limit deskew angle - avoid deskewing lines with a bigger abs(angle)
    :param skew_angle: skew the image in degrees (- for clockwise, + for counter clockwise)
    :param grayscale: convert the image to grayscale
    :return: the extracted image
    """
    if grayscale:
        background = WHITE
    else:
        background = (WHITE, WHITE, WHITE)

    if extraction_mode == Extraction.boundingRect:
        return extract_bbox_image(img, bbox=bbox)
    elif extraction_mode == Extraction.polygon:
        return extract_polygon_image(img, polygon=polygon, bbox=bbox)
    elif extraction_mode == Extraction.min_area_rect:
        return extract_min_area_rect_image(img, polygon=polygon)
    elif extraction_mode.extra == DESKEW:
        assert (
            max_deskew_angle is not None and max_deskew_angle > 0
        ), "max_deskew_angle can't be None and must be greater than 0"

        if extraction_mode == Extraction.deskew_polygon:
            extracted_img = extract_polygon_image(img, polygon=polygon, bbox=bbox)
        elif extraction_mode == Extraction.deskew_min_area_rect:
            extracted_img = extract_min_area_rect_image(img, polygon=polygon)
        else:
            raise ValueError(f"Unsupported deskew extraction mode: {extraction_mode}")

        return deskew_image(extracted_img, polygon, max_deskew_angle, background)

    elif extraction_mode.extra == SKEW:
        assert (
            skew_angle is not None and skew_angle != 0
        ), "skew_angle can't be None and must not equal 0"

        if extraction_mode == Extraction.skew_polygon:
            extracted_img = extract_polygon_image(img, polygon=polygon, bbox=bbox)

        elif extraction_mode == Extraction.skew_min_area_rect:
            extracted_img = extract_min_area_rect_image(img, polygon=polygon)
        else:
            raise ValueError(f"Unsupported skew extraction mode: {extraction_mode}")

        return rotate_and_trim(extracted_img, skew_angle, background)

    else:
        raise ValueError(f"Unsupported extraction mode: {extraction_mode}")


def extract_from_img_file(
    in_file: Path,
    out_file: Path,
    polygon_file: Path,
    extraction_mode: Extraction = Extraction.boundingRect,
    max_deskew_angle: int = None,
    skew_angle: int = None,
    grayscale: bool = False,
    scale_x: float = None,
    scale_y_top: float = None,
    scale_y_bottom: float = None,
    # **kwargs,
) -> bool:
    img = read_img(in_file, grayscale)
    poly_json = json.loads(polygon_file.read_text())
    polygon = np.asarray(poly_json)
    if scale_x or scale_y_top or scale_y_bottom:
        max_height = img.shape[0]
        max_width = img.shape[1]
        scale_x = scale_x or 1.0
        scale_y_top = scale_y_top or 1.0
        scale_y_bottom = scale_y_bottom or 1.0
        polygon = resize(
            polygon, max_width, max_height, scale_x, scale_y_top, scale_y_bottom
        )
        polygon = np.asarray(polygon)
    rect = polygon_to_bbox(polygon)
    extracted_img = extract(
        img, polygon, rect, extraction_mode, max_deskew_angle, skew_angle, grayscale
    )
    return save_img(out_file, extracted_img)


def read_img(path: Union[Path, str], grayscale=False) -> np.ndarray:
    if grayscale:
        _cv2_flag = cv2.IMREAD_GRAYSCALE
    else:
        _cv2_flag = cv2.IMREAD_COLOR
    img = np.asarray(cv2.imread(str(path), _cv2_flag))
    return img


def save_img(path: Union[Path, str], img: np.ndarray) -> bool:
    path = Path(path)
    path.parent.mkdir(parents=True, exist_ok=True)
    success = cv2.imwrite(str(path), img)
    if not success:
        logger.info(f"Saving image to {path} failed")
    return success



# -*- coding: utf-8 -*-
import argparse
from pathlib import Path

from line_image_extractor.extractor import extract_from_img_file
from line_image_extractor.image_utils import Extraction


def create_cli_parser():

    parser = argparse.ArgumentParser(
        description="Script to extract line polygon images with different methods",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )

    parser.add_argument("-i", "--in_file", type=Path, required=True, help="Input image")

    parser.add_argument(
        "-o", "--out_file", type=Path, required=True, help="Output image"
    )

    parser.add_argument(
        "-p",
        "--polygon_file",
        type=Path,
        required=True,
        help="Input file containing a polygon (2D list of ints)",
    )

    parser.add_argument(
        "-e",
        "--extraction_mode",
        type=lambda x: Extraction[x],
        default=Extraction.boundingRect,
        help=f"Mode for extracting the line images: {[e.name for e in Extraction]}",
    )

    parser.add_argument(
        "--max_deskew_angle",
        type=int,
        default=45,
        help="Maximum angle by which deskewing is allowed to rotate the line image. "
        "If the angle determined by deskew tool is bigger than max "
        "then that line won't be deskewed/rotated.",
    )

    parser.add_argument(
        "--skew_angle",
        type=int,
        default=0,
        help="Angle by which the line image will be rotated. Useful for data augmnetation"
        " - creating skewed text lines for a more robust model."
        " Only used with skew_* extraction modes.",
    )

    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument(
        "--grayscale",
        action="store_true",
        dest="grayscale",
        help="Convert images to grayscale (By default grayscale)",
    )
    group.add_argument(
        "--color", action="store_false", dest="grayscale", help="Use color images"
    )
    group.set_defaults(grayscale=True)

    parser.add_argument(
        "--scale_x",
        type=float,
        default=None,
        help="Ratio of how much to scale the polygon horizontally (1.0 means no rescaling)",
    )
    parser.add_argument(
        "--scale_y_top",
        type=float,
        default=None,
        help="Ratio of how much to scale the polygon vertically on the top (1.0 means no rescaling)",
    )

    parser.add_argument(
        "--scale_y_bottom",
        type=float,
        default=None,
        help="Ratio of how much to scale the polygon vertically on the bottom (1.0 means no rescaling)",
    )

    return parser


def main():
    parser = create_cli_parser()
    args = parser.parse_args()

    extract_from_img_file(**vars(args))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pathlib import Path

from setuptools import find_packages, setup


def parse_requirements():
    path = Path(__file__).parent.resolve() / "requirements.txt"
    assert path.exists(), f"Missing requirements: {path}"
    return list(map(str.strip, path.read_text().splitlines()))


setup(
    name="teklia-line-image-extractor",
    version=open("VERSION").read(),
    description="A tool for extracting a text line image from the contour with different methods",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Martin Maarand",
    author_email="maarand@teklia.com",
    install_requires=parse_requirements(),
    entry_points={
        "console_scripts": ["line-image-extractor=line_image_extractor.main:main"]
    },
    packages=find_packages(),
    include_package_data=True,
    keywords="line transformation image extraction",
    url="https://gitlab.com/teklia/line_image_extractor",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Scientific/Engineering :: Image Recognition",
    ],
)
